package com.faraphel.tasks_valider.connectivity.task

import okhttp3.HttpUrl
import okhttp3.MediaType.Companion.toMediaType
import okhttp3.OkHttpClient
import okhttp3.RequestBody.Companion.toRequestBody


/**
 * A client to handle the room connection.
 * @param address the address of the server
 * @param port the port of the server
 * @param baseCookies list of cookies to use (optional)
 */
class TaskClient(
    private val address: String,
    private val port: Int,
    private val baseCookies: List<okhttp3.Cookie> = listOf()
) {
    private val baseUrl = "http://$address:$port"
    private val client = OkHttpClient().newBuilder().cookieJar(
        // TODO(Faraphel): should be moved into another object
        object : okhttp3.CookieJar {
            private val cookies = baseCookies.toMutableList()  ///< list of cookies

            override fun loadForRequest(url: HttpUrl): List<okhttp3.Cookie> {
                return this.cookies
            }
            override fun saveFromResponse(url: HttpUrl, cookies: List<okhttp3.Cookie>) {
                this.cookies.addAll(cookies)
            }
        }
    ).build()

    // TODO(Faraphel): automatically convert content to the correct type ?

    /**
     * Return a basic request to the server
     * @param endpoint the endpoint of the server
     */
    private fun baseRequestBuilder(endpoint: String): okhttp3.Request.Builder =
        okhttp3.Request.Builder().url("$baseUrl/$endpoint")

    /**
     * Run a HEAD request
     * @param endpoint the endpoint of the server
     */
    fun head(endpoint: String): okhttp3.Request =
        this.baseRequestBuilder(endpoint).head().build()

    /**
     * Run a GET request
     * @param endpoint the endpoint of the server
     */
    fun get(endpoint: String): okhttp3.Response =
        this.client.newCall(
            this.baseRequestBuilder(endpoint)
                .get()
            .build()
        ).execute()

    /**
     * Run a POST request
     * @param endpoint the endpoint of the server
     * @param content the content of the request
     * @param type the type of the content
     */
    fun post(endpoint: String, content: String, type: String = "text/plain"): okhttp3.Response =
        this.client.newCall(
            this.baseRequestBuilder(endpoint)
                .post(content.toRequestBody(type.toMediaType()))
            .build()
        ).execute()

    /**
     * Run a PATCH request
     * @param endpoint the endpoint of the server
     * @param content the content of the request
     * @param type the type of the content
     */
    fun patch(endpoint: String, content: String, type: String = "text/plain"): okhttp3.Response =
        this.client.newCall(
            this.baseRequestBuilder(endpoint)
                .patch(content.toRequestBody(type.toMediaType()))
                .build()
        ).execute()

    /**
     * Run a DELETE request
     * @param endpoint the endpoint of the server
     * @param content the content of the request
     * @param type the type of the content
     */
    fun delete(endpoint: String, content: String, type: String = "text/plain"): okhttp3.Response =
        this.client.newCall(
            this.baseRequestBuilder(endpoint)
                .delete(content.toRequestBody(type.toMediaType()))
                .build()
        ).execute()
}